# -*- encoding: utf-8 -*-
from django.test import TestCase

def test_forms():
    """
    Forms doctests
    >>> from django.conf import settings

    # Ugly, but necessary for testing purposes
    >>> settings.CUSTOM_USER_MODEL = None

    >>> from django.contrib.auth.models import User
    >>> from email_auth.forms import AuthenticationForm
    >>> user = User.objects.create_user("jsmith", "jsmith@example.com", "test123")

    # The user cannot log in with username, even correct

    >>> data = {
    ...     'username': 'jsmith',
    ...     'password': 'test123',
    ... }

    >>> form = AuthenticationForm(None, data)
    >>> form.is_valid()
    False
    >>> form.non_field_errors()
    [u'Please enter a correct email and password. Note that both fields are case-sensitive.']

    # The user submits an invalid email.

    >>> data = {
    ...     'email': 'jsmith',
    ...     'password': 'test123',
    ... }

    >>> form = AuthenticationForm(None, data)
    >>> form.is_valid()
    False
    >>> form.non_field_errors()
    [u'Please enter a correct email and password. Note that both fields are case-sensitive.']

    # The user is inactive.

    >>> data = {
    ...     'email': 'jsmith@example.com',
    ...     'password': 'test123',
    ... }
    >>> user.is_active = False
    >>> user.save()
    >>> form = AuthenticationForm(None, data)
    >>> form.is_valid()
    False
    >>> form.non_field_errors()
    [u'This account is inactive.']

    >>> user.is_active = True
    >>> user.save()

    # The success case

    >>> form = AuthenticationForm(None, data)
    >>> form.is_valid()
    True
    >>> form.non_field_errors()
    []

    """
    pass

from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured

from email_auth.backends import EmailBackend

class BackendTests(TestCase):
    def test_user_class(self):
        settings.CUSTOM_USER_MODEL = None
        backend = EmailBackend()
        usermodel = backend.user_class
        self.assertEquals(usermodel, User)

        settings.CUSTOM_USER_MODEL = 'myapp.foobar'
        backend = EmailBackend()
        f = lambda x: x.user_class
        self.assertRaises(ImproperlyConfigured, f, backend)

    def test_authenticate(self):
        settings.CUSTOM_USER_MODEL = None
        user = User.objects.create_user("jsmith", "jsmith@example.com", "test123")
        backend = EmailBackend()
        u = backend.authenticate("jsmith@example.com","badpassword")
        self.assertEqual(u, None)
        u = backend.authenticate("jsmith","test123")
        self.assertEqual(u, None)
        u = backend.authenticate("jsmith@example.com","test123")
        self.assertEqual(u, user)

    def test_get_user(self):
        settings.CUSTOM_USER_MODEL = None
        user = User.objects.create_user("jsmith", "jsmith@example.com", "test123")
        user.save()
        backend = EmailBackend()
        u = backend.get_user(user.pk)
        self.assertEqual(u, user)
        u = backend.get_user(0)
        self.assertEqual(u, None)

from django.http import HttpResponse

from email_auth.views import login as email_login

def fakeview(request):
    return HttpResponse("Ok logged in.")

class GeneralTests(TestCase):
    urls= 'email_auth.test_urls'

    def test_login_view(self):
        response = self.client.get('/login/')
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response,'registration/login.html')

    def test_admin_access(self):
        response = self.client.get('/admin/')
        self.assertRedirects(response,'/login/?next=%2Fadmin%2F',302,200)

    def test_login_then_logout(self):
        settings.CUSTOM_USER_MODEL = None
        settings.REDIRECT_FIELD_NAME = 'next'
        user = User.objects.create_user("jsmith", "jsmith@example.com", "test123")
        response = self.client.post('/login/?next=%2Flogged_in%2F',
                {'email':'jsmith@example.com', 'password':'test123'})
        self.assertRedirects(response,'/logged_in/',302,200)
        response = self.client.get('/logout/')
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response,'registration/logged_out.html')



# -*- encoding: utf-8 -*-

import datetime
import urlparse
from urllib import quote

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.utils.translation import ugettext as _
from django.utils.encoding import iri_to_uri
from django.views.decorators.cache import never_cache
from django.contrib.sites.models import Site
from django.template import RequestContext
from django.dispatch import Signal

from email_auth.forms import AuthenticationForm

user_logged_in = Signal(providing_args=['request',])
user_logged_out = Signal(providing_args=['request',])

def login(request, template_name='registration/login.html',
          redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Displays the login form, handles the email-based login action.
    May set a "remember me" cookie.
    Adapted from django.contrib.auth.views.login
    """
    from base64 import encodestring, decodestring
    redirect_to = request.REQUEST.get('next', '')
    if request.method == "POST":
        form = AuthenticationForm(data=request.POST)
        if form.is_valid():
            netloc = urlparse.urlparse(redirect_to)[1]

            # Use default setting if redirect_to is empty
            if not redirect_to:
                redirect_to = settings.LOGIN_REDIRECT_URL

            # Heavier security check -- don't allow redirection to a different
            # host.
            elif netloc and netloc != request.get_host():
                redirect_to = settings.LOGIN_REDIRECT_URL

            from django.contrib.auth import login
            login(request, form.get_user())
            if request.session.test_cookie_worked():
                request.session.delete_test_cookie()
            response = HttpResponse()
            # handle "remember me" cookie
            # effacer le cookie s'il existe
            response.delete_cookie('django_email_auth')
            if form.cleaned_data['remember']:
                try:
                    cookie_data = encodestring('%s:%s' %
                        (form.cleaned_data['email'],
                        form.cleaned_data['password']))
                    max_age = 30*24*60*60
                    expires = datetime.datetime.strftime(
                        datetime.datetime.utcnow() +
                        datetime.timedelta(seconds=max_age),
                        "%a, %d-%b-%Y %H:%M:%S GMT"
                        )
                    response.set_cookie('django_email_auth',
                            cookie_data, max_age=max_age, expires=expires)
                except UnicodeEncodeError:
                    pass
            # send signal "user just logged in"
            user_logged_in.send(sender=request.user, request=request)
            # retourner à la vue appelante
            if redirect_to:
                response.status_code = 302
                response['Location'] = iri_to_uri(redirect_to)
                return response
            else:
                response.status_code = 302
                response['Location'] = settings.LOGIN_REDIRECT_URL
                return response
    else:
        # get login cookie if any
        if 'django_email_auth' in request.COOKIES:
            cookie_data = decodestring(request.COOKIES['django_email_auth'])
            try:
                e,p = cookie_data.split(':')
            except ValueError:
                e,p = (None,None)
            form = AuthenticationForm(request, 
                    {'email': e, 'password': p, 'remember': True})
        else:
            form = AuthenticationForm(request)
    request.session.set_test_cookie()
    if Site._meta.installed:
        current_site = Site.objects.get_current()
    else:
        current_site = RequestSite(request)
    return render_to_response(template_name, {
        'form': form,
        redirect_field_name: redirect_to,
        'site': current_site,
        'site_name': current_site.name,
    }, context_instance=RequestContext(request))
login = never_cache(login)

def logout(request, next_page=None,
           template_name='registration/logged_out.html',
           redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Logs out the user and displays 'You are logged out' message.
    Sends the user_logged_out signal.
    """
    from django.contrib.auth import logout
    user_was = request.user
    logout(request)
    user_logged_out.send(sender=user_was, request=request)
    if next_page is None:
        redirect_to = request.REQUEST.get(redirect_field_name, '')
        if redirect_to:
            return HttpResponseRedirect(redirect_to)
        else:
            return render_to_response(template_name, {
                'title': _('Logged out')
            }, context_instance=RequestContext(request))
    else:
        # Redirect to this page until the session has been cleared.
        return HttpResponseRedirect(next_page or request.path)




# -*- encoding: utf-8 -*-
"""
Email authentication backend
As users are searched on email, indexing the user table
on email in the db might be a good idea.
"""

from django.conf import settings
from django.contrib.auth.backends import ModelBackend
from django.core.exceptions import ImproperlyConfigured
from django.db.models import get_model
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

class EmailBackend(ModelBackend):
    """
    Authenticate user against email,password credentials
    The user class might be :
    - django.contrib.auth.models.User (default)
    - a custom User class inheriting from django.contrib.auth.models.User
    A custom User class is declared through CUSTOM_USER_MODEL setting :
    CUSTOM_USER_MODEL = 'yourapp.YourCustomUser' 
    """
    def authenticate(self, email=None, password=None):
        try:
            user = self.user_class.objects.get(email=email)
        except self.user_class.DoesNotExist:
            return None
        if user.check_password(password):
            return user
        return None

    def get_user(self, user_id):
        try:
            return self.user_class.objects.get(pk=user_id)
        except self.user_class.DoesNotExist:
            return None

    @property
    def user_class(self):
        if not hasattr(self, '_user_class'):
            try:
                user_model = settings.CUSTOM_USER_MODEL
                self._user_class = get_model(*user_model.split('.', 2))
                if not self._user_class:
                    raise ImproperlyConfigured(
                            _('Could not get custom user model %s') % user_model)
                return self._user_class
            except AttributeError:
                return User
        return self._user_class




# -*- encoding: utf-8 -*-

from django import forms
from django.contrib.auth import authenticate
from django.contrib.auth.forms import AuthenticationForm as OriginalAuthForm
from django.utils.translation import ugettext_lazy as _

class AuthenticationForm(OriginalAuthForm):
    """
    Form for authenticating users on email/password credentials.
    The difference with the original Form from contrib.auth.forms
    is the use of the email field instead of username.
    See doctests in the tests.py module.
    """
    # username has to be there to override required field from orig auth form
    username = forms.CharField(required=False)
    email = forms.EmailField(label=_("Email"), max_length=75)
    remember = forms.BooleanField( label=_("Remember me on this computer"), 
            required=False)

    def clean(self):
        email = self.cleaned_data.get('email')
        password = self.cleaned_data.get('password')

        if email and password:
            self.user_cache = authenticate(email=email, password=password)
            if self.user_cache is None:
                raise forms.ValidationError(
                    _("Please enter a correct email and password. Note that both fields are case-sensitive."))
            elif not self.user_cache.is_active:
                raise forms.ValidationError(_("This account is inactive."))
        else:
            raise forms.ValidationError(
                _("Please enter a correct email and password. Note that both fields are case-sensitive."))

        # TODO: determine whether this should move to its own method.
        if self.request:
            if not self.request.session.test_cookie_worked():
                raise forms.ValidationError(
                    _("Your Web browser doesn't appear to have cookies enabled. Cookies are required for logging in."))

        return self.cleaned_data




# -*- encoding: utf-8 -*-

"""
These are for testing purposes only.
"""

from django.conf.urls.defaults import *

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
    (r'^login/$', 'email_auth.views.login'),
    (r'^logout/$', 'email_auth.views.logout'),
    (r'^logged_in/$','email_auth.tests.fakeview'),
)




# -*- encoding: utf-8 -*-

from urllib import urlencode

from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.core.urlresolvers import reverse
from django.contrib.auth.views import login
from django.contrib import messages
from django.utils.translation import ugettext as _

from email_auth.views import login as email_login

class EmailAuthMiddleware(object):
    """
    Middleware making sure that any auth request, including from
    the admin, uses the email auth backend.
    """

    def process_view(self, request, view_func, view_args, view_kwargs):
        """
        Forwards any unauthenticated requests to our login url, 
        be it calls to django.contrib.auth.views.login or admin requests.
        """
        if view_func == login:
            return email_login(request, *view_args, **view_kwargs)

        if not view_func.__module__.startswith('django.contrib.admin'):
            return None

        # Now we know for sure the view_func belongs to the admin

        if request.user.is_authenticated():
            if request.user.is_staff:
                return None
            elif settings.LOGIN_REDIRECT_URL:
                message = _(
                    "Sorry, you are not allowed to access to %s" % request.path)
                messages.add_message(request, messages.WARNING, message)
                return HttpResponseRedirect(settings.LOGIN_REDIRECT_URL)
            else:
                error = (
                    _('<h1>Forbidden</h1><p>You do not have staff privileges.</p>'))
                return HttpResponseForbidden(error)

        params = urlencode({REDIRECT_FIELD_NAME: request.get_full_path()})
        url = reverse(email_login)
        url = reverse(getattr(settings, 'LOGIN_URL_MAP', email_login))
        return HttpResponseRedirect(url + '?' + params)



from distutils.core import setup

version = __import__('email_auth').VERSION

setup(
        name='django_email_auth',
        version=version,
        author='Jean-Charles Bagneris',
        author_email='jcbagneris@gmail.com',
        description='Yet another email auth system for the Django web framework',
        url="http://github.com/jcbagneris/django_email_auth",
        packages=['email_auth',],
        include_package_data=True,
        license='BSD',
        long_description=open('README.txt').read(),
        classifiers=[
            "Development Status :: 4 - Beta",
            "Environment :: Web Environment",
            "Framework :: Django",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: BSD License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 2",
        ],
)

